/* ezio.c -- protocol implementation
 *
 * Copyright (c) 2010, Angelo Haller <angelo@szanni.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "ezio.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
/* Fix missing usb.h includes */
#include <sys/types.h>
#include <linux/types.h>
#include <usb.h>


static int vendor_id  = 0x056d;
static int product_id = 0x0002;

struct usb_dev_handle *device_handle;

/* Search for supported usb devices */
static int
usb_find_device ()
{
  usb_find_busses ();
  usb_find_devices ();

  struct usb_bus* bus;

  for (bus = usb_get_busses (); bus; bus = bus->next)
    {
      struct usb_device *device;

      for (device = bus->devices; device; device=device->next)
        {
          if ((device->descriptor.idVendor == vendor_id)
               && (device->descriptor.idProduct == product_id))
            {
              /* Open device */
              if (!(device_handle = usb_open (device)))
                {
                  puts ("Open failed!");
                  return -1;
                }
              else
                {
                  return 0;
                }
            }
        }
    }
    puts ("No device found!");
    return -1;
}

/* Send signal to monitor */
static int
usb_send_signal (const struct usb_ctrl_setup msg, char bytes[3])
{
  /* TODO send wakeup signal */
  if (usb_control_msg (device_handle, msg.bRequestType, msg.bRequest, msg.wValue, msg.wIndex, bytes, msg.wLength, 1000) < 0)
    {
      puts ("Error while sending control msg");
      return -1;
    }
  return 0;
}

int
ezio_init ()
{
  usb_init ();

  if (usb_find_device () < 0)
    return -1;

  //claim interface
  if (usb_claim_interface (device_handle, 0) < 0)
    {
      //claim by force
      if ((usb_detach_kernel_driver_np (device_handle, 0) < 0)
           && (usb_claim_interface (device_handle, 0) < 0))
        {
          puts ("Claiming interface failed!");
          return -1;
        }
      puts ("Claimed interface by force!");
    }
  return 0;
}

void
ezio_exit ()
{
  usleep (100000);
  usb_release_interface (device_handle, 0);
  usb_close (device_handle);
}

int
ezio_list_devices (void)
{
  static const struct usb_ctrl_setup msg = {0xa1, 1, 0x32c, 0, 9};
  char bytes[10] = {[9]='\0'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while getting device names");
      return -1;
    }

  char *val = bytes;
  while (*val == ',')
    ++val;

  printf ("%s\n", val);

  return 0;
}

int
ezio_set_mode (int mode)
{
  if (mode < 1 || mode > 5)
    {
      puts ("Mode out of range [1, 5]");
      return -1;
    }

  static const struct usb_ctrl_setup msg = {0x21, 9, 0x305, 0, 3};
  char bytes[3] = {'\x05', --mode, '\x00'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while setting mode");
      return -1;
    }

  return 0;
}

int
ezio_set_brightness (int brightness)
{
  if (brightness < 0 || brightness > 100)
    {
      puts ("Brightness out of range [0, 100]");
      return -1;
    }

  static const struct usb_ctrl_setup msg = {0x21, 9, 0x347, 0, 3};
  char bytes[3] = {'\x47', brightness*2, '\x00'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while setting brightness");
      return -1;
    }

  return 0;
}

int
ezio_set_contrast (int contrast)
{
  if (contrast < 0 || contrast > 100)
    {
      puts ("Contrast out of range [0, 100]");
      return -1;
    }

  static const struct usb_ctrl_setup msg = {0x21, 9, 0x353, 0, 3};
  char bytes[3] = {'\x53', contrast*14/10, '\x00'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while setting contrast");
      return -1;
    }

  return 0;
}

int
ezio_set_saturation (int saturation)
{
  if (saturation < -128 || saturation > 127)
    {
      puts ("Contrast out of range [0, 255]");
      return -1;
    }

  static const struct usb_ctrl_setup msg = {0x21, 9, 0x324, 0, 3};
  char bytes[3] = {'\x24', saturation+128, '\x00'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while setting contrast");
      return -1;
    }

  return 0;
}

int
ezio_set_volume (int volume)
{
  if (volume < 0 || volume > 20)
    {
      puts ("Volume out of range [0, 20]");
      return -1;
    }

  static const struct usb_ctrl_setup msg = {0x21, 9, 0x354, 0, 3};
  char bytes[3] = {'\x54', volume, '\x00'};

  if (usb_send_signal (msg, bytes) < 0)
    {
      puts ("Error while setting volume");
      return -1;
    }

  return 0;
}

