/* ezioctrl -- userspace driver to control EIZO monitor settings
 *
 * Copyright (c) 2010, Angelo Haller <angelo@szanni.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "ezio.h"
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

/* Print version and exit program */
static void
version ()
{
  puts (PACKAGE_STRING "\n\
Copyright (c) 2010 Angelo Haller <angelo@szanni.org>\n\
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\n\
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\n\
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE\n\
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE\n\
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR\n\
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF\n\
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS\n\
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN\n\
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)\n\
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE\n\
POSSIBILITY OF SUCH DAMAGE.");

  exit (0);
}

/* Print help and exit program */
static void
help ()
{
  puts ("Usage: " PACKAGE_NAME " [OPTION]... [DEVICE]...\n\
Control EIZO monitor settings.\n\
\n\
  -b, --brightness BRIGHTNESS   set brightness of current mode to BRIGHTNESS\n\
  -c, --contrast CONTRAST       set contrast of current mode to CONTRAST\n\
  -m, --mode MODE               set mode to MODE\n\
  -s, --saturation SATURATION   set saturation of current mode to SATURATION\n\
  -v, --volume VOLUME           set volume to VOLUME\n\
\n\
  -l, --list                    list device names\n\
\n\
      --help      display this help and exit\n\
      --version   output version information and exit\n\
\n\
Report bugs to: " PACKAGE_BUGREPORT);

  exit (0);
}

int
main (int argc, char **argv)
{
  _Bool ezio_initialized = 0;
#define EZIO_LAZY_INIT_OR_BREAK if (!ezio_initialized &&\
                                    !(ezio_initialized = !ezio_init ())) break
  int c;

  while (1)
    {
      static struct option long_options[] =
        {
          /* These options don't set a flag.
             We distinguish them by their indices. */
          { "version", no_argument, 0, 'V' },
          { "help", no_argument, 0, 'h' },
          { "mode", required_argument, 0, 'm' },
          { "volume", required_argument, 0, 'v' },
          { "brightness", required_argument, 0, 'b' },
          { "contrast", required_argument, 0, 'c' },
          { "saturation", required_argument, 0, 's' },
          { "list", no_argument, 0, 'l' },
          { 0, 0, 0, 0 }
        };
      /* getopt_long stores the option index here. */
      int option_index = 0;

      c = getopt_long (argc, argv, "v:m:b:c:s:l", long_options, &option_index);

      /* Detect the end of the options. */
      if (c == -1)
        break;

      switch (c)
        {
        case 0:
          /* If this option set a flag, do nothing else now. */
          if (long_options[option_index].flag != 0)
            break;
          printf ("option %s", long_options[option_index].name);
          if (optarg)
            printf (" with arg %s", optarg);
          printf ("\n");
          break;

        case 'V':
          version ();
          break;

        case 'm':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_set_mode (atoi (optarg));
          break;

        case 'v':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_set_volume (atoi (optarg));
          break;

        case 'b':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_set_brightness (atoi (optarg));
          break;

        case 'c':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_set_contrast (atoi (optarg));
          break;

        case 's':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_set_saturation (atoi (optarg));
          break;

        case 'l':
          EZIO_LAZY_INIT_OR_BREAK;
          ezio_list_devices ();
          break;

        case '?':
          /* getopt_long already printed an error message. */
          printf ("Try `%s --help' for more information.\n", argv[0]);
          break;

        default:
          help ();
        }
    }

  /* Print any remaining command line arguments (not options). */
  if (optind < argc)
    {
      printf ("non-option ARGV-elements: ");
      while (optind < argc)
        printf ("%s ", argv[optind++]);
      putchar ('\n');
    }

  if (ezio_initialized)
    ezio_exit ();

  exit (0);
}

